<?php
/**
 * Copyright © Lyra Network.
 * This file is part of EpayNC plugin for Magento. See COPYING.md for license details.
 *
 * @author    Lyra Network (https://www.lyra.com/)
 * @copyright Lyra Network
 * @license   https://opensource.org/licenses/osl-3.0.php Open Software License (OSL 3.0)
 */

class Lyra_Epaync_Model_Payment_Sofort extends Lyra_Epaync_Model_Payment_Abstract
{
    protected $_code = 'epaync_sofort';
    protected $_formBlockType = 'epaync/sofort';

    protected $_canUseInternal = false;

    protected $_currencies = array('EUR', 'CHF', 'GBP', 'PLN');

    protected  function _setExtraFields($order)
    {
        // Override with Sofort banking payment card.
        $this->_epayncRequest->set('payment_cards', 'SOFORT_BANKING');
    }

    /**
     * Assign data to info model instance
     *
     * @param  mixed $data
     * @return Mage_Payment_Model_Info
     */
    public function assignData($data)
    {
        $info = $this->getInfoInstance();

        // Init all payment data.
        $info->setCcType(null)
            ->setCcLast4(null)
            ->setCcNumber(null)
            ->setCcCid(null)
            ->setCcExpMonth(null)
            ->setCcExpYear(null)
            ->setAdditionalData(null);

        return $this;
    }

    /**
     * To check billing country is allowed for SOFORT banking payment method.
     *
     * @return bool
     */
    public function canUseForCountry($country)
    {
        $availableCountries = Mage::getModel('epaync/source_sofort_availableCountries')->getCountryCodes();

        if ($this->getConfigData('allowspecific') == 1) {
            $availableCountries = explode(',', $this->getConfigData('specificcountry'));
        }

        return in_array($country, $availableCountries);
    }
}
